@@ -42,6 +42,7 @@
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <libhandy-1/handy.h>
 
 #ifdef ENABLE_INTROSPECTION
 #include <girepository.h>
@@ -405,6 +406,16 @@ quit_activated (GSimpleAction *action,
   g_application_quit (G_APPLICATION (app));
 }
 
+static void
+upload_file_activated (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       user_data)
+{
+  GtrApplication *app = GTR_APPLICATION (user_data);
+  GtrApplicationPrivate *priv = gtr_application_get_instance_private (app);
+  gtr_upload_file_dialog (NULL, priv->active_window);
+}
+
 static void
 saveas_activated (GSimpleAction *action,
                 GVariant      *parameter,
@@ -576,6 +587,8 @@ static GActionEntry app_entries[] = {
   { "save", save_activated, NULL, NULL, NULL },
   { "saveas", saveas_activated, NULL, NULL, NULL },
 
+  { "upload_file", upload_file_activated, NULL, NULL, NULL },
+
   { "open", open_activated, NULL, NULL, NULL },
   { "dl", dl_activated, NULL, NULL, NULL },
 
@@ -642,6 +655,7 @@ gtr_application_startup (GApplication *application)
 
   G_APPLICATION_CLASS (gtr_application_parent_class)->startup (application);
 
+  hdy_init();
   g_set_application_name (_("Translation Editor"));
   gtk_window_set_default_icon_name (PACKAGE_APPID);
 
@@ -663,6 +677,7 @@ gtr_application_startup (GApplication *application)
   set_kb (application, "app.dl", "<Ctrl>d");
   set_kb (application, "app.save", "<Ctrl>s");
   set_kb (application, "app.saveas", "<Ctrl><Shift>s");
+  set_kb (application, "app.upload_file", "<Ctrl>b");
   set_kb (application, "app.preferences", "<Ctrl>p");
   set_kb (application, "app.quit", "<Primary>q");
 
