@@ -42,7 +42,9 @@ typedef struct
   GtkWidget *select_box;
   GtkWidget *open_button;
   GtkWidget *load_button;
+  GtkWidget *reserve_button;
   GtkWidget *stats_label;
+  GtkWidget *module_state_label;
   GtkWidget *file_label;
   GtkWidget *instructions;
 
@@ -54,8 +56,9 @@ typedef struct
   gchar *selected_team;
   gchar *selected_module;
   gchar *selected_branch;
-  const gchar *selected_domain;
-  const gchar *file_path;
+  gchar *selected_domain;
+  gchar *file_path;
+  gchar *module_state;
 
   GtrWindow *main_window;
 } GtrDlTeamsPrivate;
@@ -71,6 +74,7 @@ static void team_add_cb (GtkButton *btn, GtrDlTeams *self);
 static void gtr_dl_teams_save_combo_selected (GtkWidget *widget, GtrDlTeams *self);
 static void gtr_dl_teams_load_po_file (GtkButton *button, GtrDlTeams *self);
 static void gtr_dl_teams_get_file_info (GtrDlTeams *self);
+static gboolean gtr_dl_teams_reserve_for_translation (GtkWidget *button, GtrDlTeams *self);
 
 static void
 gtr_dl_teams_list_add (JsonArray *array,
@@ -175,6 +179,7 @@ gtr_dl_teams_load_module_details_json (GtkWidget  *widget,
   GtkWidget *dialog;
 
   gtk_widget_hide (priv->file_label);
+  gtk_widget_hide (priv->module_state_label);
   gtk_widget_show (priv->instructions);
   gtk_label_set_text (GTK_LABEL (priv->stats_label), "");
 
@@ -355,6 +360,7 @@ gtr_dl_teams_get_file_info (GtrDlTeams *self)
   JsonObject *stats_object;
   const char *format;
   char *markup;
+  g_autofree char *module_state = NULL;
   GtkWidget *dialog;
 
   /* API endpoint: modules/[module]/branches/[branch]/domains/[domain]/languages/[team] */
@@ -404,6 +410,14 @@ gtr_dl_teams_get_file_info (GtrDlTeams *self)
       return;
     }
 
+  priv->module_state = g_strdup (json_object_get_string_member (object, "state"));
+
+  if (!priv->module_state)
+    {
+      gtk_label_set_text (GTK_LABEL (priv->module_state_label), _("No module state found."));
+      gtk_widget_show (priv->module_state_label);
+    }
+
   /* Get file statistics and show them to the user */
   stats_node = json_object_get_member (object, "statistics");
   stats_object = json_node_get_object (stats_node);
@@ -417,10 +431,24 @@ gtr_dl_teams_get_file_info (GtrDlTeams *self)
   gtk_label_set_markup (GTK_LABEL (priv->stats_label), markup);
   gtk_label_set_text (GTK_LABEL (priv->file_label), g_strconcat("File: ", strrchr (priv->file_path, '/') + 1, NULL));
   gtk_widget_show (priv->file_label);
-
+  module_state = g_strdup_printf (_("The current state is: %s"), priv->module_state);
+  gtk_label_set_text (GTK_LABEL (priv->module_state_label), module_state);
+  gtk_widget_show (priv->module_state_label);
   /* Enable (down)load button */
   gtk_widget_set_sensitive (priv->load_button, TRUE);
 
+  /* Enable the reserve button if a module's state is either None or Translated or ToReview */
+  if (strcmp(priv->module_state, "None") == 0 ||
+      strcmp(priv->module_state, "Translated") == 0 ||
+      strcmp(priv->module_state, "ToReview") == 0)
+    {
+      gtk_widget_set_sensitive (priv->reserve_button, TRUE);
+    }
+  else
+    {
+      gtk_widget_set_sensitive (priv->reserve_button, FALSE);
+    }
+
   g_free (markup);
 }
 
@@ -436,16 +464,38 @@ gtr_dl_teams_load_po_file (GtkButton *button, GtrDlTeams *self)
   GOutputStream *output = NULL;
   gsize bytes = 0;
   GtkWidget *dialog;
+  gboolean ret = FALSE;
+  int file_index = 0;
   const char *dest_dir = g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD);
+  g_autofree char *basename = NULL;
+  g_autofree char *filename = NULL;
   g_autofree char *file_path = NULL;
   g_autoptr(GFile) dest_file = NULL;
+  gboolean reserve_first = FALSE;
+
+  // reserve for translation first
+  reserve_first = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
+                                                (priv->reserve_button));
+  if (reserve_first)
+    {
+      ret = gtr_dl_teams_reserve_for_translation (priv->reserve_button, self);
+      if (!ret)
+        return;
+
+      if (priv->module_state)
+        g_free (priv->module_state);
+
+      // The reserve was successful, so we should change the module_state to
+      // "Translating"
+      priv->module_state = g_strdup ("Translating");
+    }
 
   /* Load the file, save as temp; path to file is https://l10n.gnome.org/[priv->file_path] */
   session = soup_session_new ();
   msg = soup_message_new ("GET", g_strconcat ("https://l10n.gnome.org", g_strcompress(priv->file_path), NULL));
   soup_session_send_message (session, msg);
 
-	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
+  if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
     {
       dialog = gtk_message_dialog_new (GTK_WINDOW (priv->main_window),
                                        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -495,10 +545,28 @@ gtr_dl_teams_load_po_file (GtkButton *button, GtrDlTeams *self)
     }
 
   /* Save file to Downloads; file basename is the part from last / character on */
-  file_path = g_strconcat ("file://", dest_dir, "/", strrchr (priv->file_path, '/'), NULL);
+  basename = g_path_get_basename (priv->file_path);
+  // Remove the extension
+  filename = g_strdup_printf ("%s.%s.%s", priv->selected_module, priv->selected_branch, basename);
+  g_free (basename);
+  basename = g_strdup (filename);
+  file_path = g_strconcat ("file://", dest_dir, "/", basename, NULL);
   dest_file = g_file_new_for_uri (file_path);
 
-  g_file_copy (tmp_file, dest_file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &error);
+  ret = g_file_copy (tmp_file, dest_file, G_FILE_COPY_NONE, NULL, NULL, NULL, &error);
+  while (!ret && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
+    {
+      g_autofree char *tmpname = gtr_utils_get_filename (filename);
+      g_free (basename);
+      g_free (file_path);
+      g_object_unref (dest_file);
+      g_clear_error (&error);
+
+      basename = g_strdup_printf ("%s (%d).po", tmpname, ++file_index);
+      file_path = g_strconcat ("file://", dest_dir, "/", basename, NULL);
+      dest_file = g_file_new_for_uri (file_path);
+      ret = g_file_copy (tmp_file, dest_file, G_FILE_COPY_NONE, NULL, NULL, NULL, &error);
+    }
 
   if (error != NULL)
     {
@@ -512,11 +580,98 @@ gtr_dl_teams_load_po_file (GtkButton *button, GtrDlTeams *self)
       return;
     }
 
-  gtr_open (dest_file, priv->main_window, &error);
+  if (gtr_open (dest_file, priv->main_window, &error)) {
+    GtrTab *tab = gtr_window_get_active_tab (priv->main_window);
+    g_autofree char *info_msg = NULL;
+    info_msg = g_strdup_printf (_("The file '%s' has been saved in %s"),
+                                basename, dest_dir);
+    gtr_tab_set_info (tab, info_msg, NULL);
+
+    GtrPo *po = gtr_tab_get_po(tab);
+    GError *po_error = NULL;
+    gtr_po_set_dl_info(po,
+                       priv->selected_team,
+                       priv->selected_module,
+                       priv->selected_branch,
+                       priv->selected_domain,
+                       priv->module_state);
+    // Save to update the headers
+    gtr_po_save_file(po, &po_error);
+  }
 
   g_object_unref (tmp_file);
 }
 
+/* Reserve for translation */
+static gboolean
+gtr_dl_teams_reserve_for_translation (GtkWidget *button, GtrDlTeams *self)
+{
+  GtrDlTeamsPrivate *priv = gtr_dl_teams_get_instance_private (self);
+  GtrProfileManager *pmanager = NULL;
+  GtrProfile *profile = NULL;
+  GtkWidget *dialog, *success_dialog;
+  g_autoptr (SoupSession) session = NULL;
+  g_autoptr (SoupMessage) msg = NULL;
+  const char *auth_token = NULL;
+  g_autofree char *auth = NULL;
+  g_autofree gchar *reserve_endpoint = NULL;
+
+  pmanager = gtr_profile_manager_get_default ();
+  profile = gtr_profile_manager_get_active_profile (pmanager);
+  auth_token = gtr_profile_get_auth_token (profile);
+  auth = g_strconcat ("Bearer ", auth_token, NULL);
+
+  /* API endpoint: modules/[module]/branches/[branch]/domains/[domain]/languages/[team]/reserve */
+  reserve_endpoint = g_strconcat ((const gchar *)API_URL,
+                                  "modules/", priv->selected_module,
+                                  "/branches/", priv->selected_branch,
+                                  "/domains/", priv->selected_domain,
+                                  "/languages/", priv->selected_team,
+                                  "/reserve", NULL);
+
+  msg = soup_message_new ("POST", reserve_endpoint);
+  soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
+  soup_message_headers_append (msg->request_headers, "Authentication", auth);
+  session = soup_session_new ();
+  soup_session_send_message (session, msg);
+
+  if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
+  {
+    dialog = gtk_message_dialog_new_with_markup (
+      GTK_WINDOW (priv->main_window),
+      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+      GTK_MESSAGE_WARNING,
+      GTK_BUTTONS_CLOSE,
+      _(
+        "An error occurred while reserving this module: %s\n"
+        "Maybe you've not configured your <i>l10n.gnome.org</i> "
+        "<b>token</b> correctly in your profile or you don't have "
+        "permissions to reserve this module."
+      ),
+      soup_status_get_phrase (msg->status_code));
+    gtk_dialog_run (GTK_DIALOG (dialog));
+    gtk_widget_destroy (dialog);
+    return FALSE;
+  }
+
+  /* Display a message if the reserve for translation operation was successful */
+  success_dialog = gtk_message_dialog_new (GTK_WINDOW (priv->main_window),
+                                           GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                           GTK_MESSAGE_INFO,
+                                           GTK_BUTTONS_OK,
+                                           _("The file '%s.%s.%s.%s' has been successfully reserved"),
+                                           priv->selected_module,
+                                           priv->selected_branch,
+                                           priv->selected_team,
+                                           priv->selected_domain);
+
+  gtk_dialog_run (GTK_DIALOG (success_dialog));
+  gtk_widget_destroy (success_dialog);
+  gtk_widget_set_sensitive (priv->reserve_button, FALSE);
+
+  return TRUE;
+}
+
 static void
 gtr_dl_teams_save_combo_selected (GtkWidget  *widget,
                                   GtrDlTeams *self)
@@ -552,7 +707,7 @@ gtr_dl_teams_save_combo_selected (GtkWidget  *widget,
     }
   else if (strcmp(name, "combo_domains") == 0)
     {
-      priv->selected_domain = gtk_combo_box_get_active_id (GTK_COMBO_BOX (widget));
+      priv->selected_domain = g_strdup (gtk_combo_box_get_active_id (GTK_COMBO_BOX (widget)));
     }
 
   /* Check if all four required values have been selected to proceed with loading PO file */
@@ -566,6 +721,14 @@ gtr_dl_teams_dispose (GObject *object)
 
   if (priv->selected_team)
     g_free (priv->selected_team);
+  if (priv->selected_module)
+    g_free (priv->selected_module);
+  if (priv->selected_branch)
+    g_free (priv->selected_branch);
+  if (priv->selected_domain)
+    g_free (priv->selected_domain);
+  if (priv->module_state)
+    g_free (priv->module_state);
 
   G_OBJECT_CLASS (gtr_dl_teams_parent_class)->dispose (object);
 }
@@ -593,7 +756,9 @@ gtr_dl_teams_class_init (GtrDlTeamsClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, select_box);
   gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, file_label);
   gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, stats_label);
+  gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, module_state_label);
   gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, load_button);
+  gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, reserve_button);
   gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, instructions);
 
   gtk_widget_class_bind_template_child_private (widget_class, GtrDlTeams, open_button);
@@ -608,8 +773,14 @@ gtr_dl_teams_init (GtrDlTeams *self)
 
   priv->main_window = NULL;
   priv->selected_team = NULL;
+  priv->selected_module = NULL;
+  priv->selected_branch = NULL;
+  priv->selected_domain = NULL;
+  priv->file_path = NULL;
+  priv->module_state = NULL;
 
   gtk_widget_set_sensitive (priv->load_button, FALSE);
+  gtk_widget_set_sensitive (priv->reserve_button, FALSE);
 
   /* Add combo boxes for DL teams and modules */
   priv->teams_combobox = GTK_WIDGET (gtr_filter_selection_new ());
@@ -697,3 +868,4 @@ team_add_cb (GtkButton   *btn,
   gtr_open_file_dialog (NULL, window);
 }
 
+
